#include "crappymodule.h"
#include "shader.h"
#include "camera.h"
#include "mesh.h"
#include "sparrowrenderer.h"
#include "scene.h"

#include "material.h"
#include <glm/ext.hpp>

void CrappyModule::renderGL(Camera* myCamera, Scene* scene)
{
    glEnable(GL_LIGHTING);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glLoadIdentity();
    glEnable(GL_LIGHT0);
    glLightfv(GL_LIGHT0, GL_AMBIENT, glm::value_ptr(glm::vec4(glm::vec3(0.1f), 1)));
    glLightfv(GL_LIGHT0, GL_POSITION, glm::value_ptr(glm::vec4(0)));
    glLightfv(GL_LIGHT0, GL_DIFFUSE, glm::value_ptr(glm::vec4(0, 0, 0, 1)));
    glLightfv(GL_LIGHT0, GL_SPECULAR, glm::value_ptr(glm::vec4(0, 0, 0, 1)));

    int i=1;
    for(SceneIterator<Light*>* lightIt = scene->getLights();
        lightIt->isValid(); lightIt->next())
    {
        Light* l = lightIt->getItem();
        glEnable(GL_LIGHT0 + i);
        glLightfv(GL_LIGHT0 + i, GL_AMBIENT, glm::value_ptr(glm::vec4(glm::vec3(0), 1)));
        if(l->isDirectionnal()){
            glLightfv(GL_LIGHT0 + i, GL_POSITION, glm::value_ptr(glm::vec4(-l->getDir(), 0)));
        }else{
            glLightfv(GL_LIGHT0 + i, GL_POSITION, glm::value_ptr(glm::vec4(l->getPos(), 1)));
        }
        glLightfv(GL_LIGHT0 + i, GL_DIFFUSE, glm::value_ptr(glm::vec4(l->getColor(), 1)));
        glLightfv(GL_LIGHT0 + i, GL_SPECULAR, glm::value_ptr(glm::vec4(l->getColor(), 1)));
        ++i;
    }

    for(SceneIterator<GeometryNode*>* geometryIt = scene->getGeometry();
        geometryIt->isValid(); geometryIt->next())
    {
        GeometryNode* node = geometryIt->getItem();
        glMatrixMode(GL_PROJECTION);
        glLoadMatrixf(glm::value_ptr(myCamera->getProjectionMatrix()));
        glMatrixMode(GL_MODELVIEW);
        if(!node->mesh->instances_offsets.empty())
        {
            for(const glm::vec3 &pos : node->mesh->instances_offsets)
            {
                glm::mat4 offsetMatrix = glm::translate(node->modelMatrix, pos);
                glm::mat4 modelViewMatrix = myCamera->getViewMatrix() * offsetMatrix;
                glLoadMatrixf(glm::value_ptr(modelViewMatrix));
                node->mesh->draw();
            }
        }
        else
        {
            glm::mat4 modelViewMatrix = myCamera->getViewMatrix() * node->modelMatrix;
            glLoadMatrixf(glm::value_ptr(modelViewMatrix));
            node->mesh->draw();
        }
    }

    glDisable(GL_LIGHTING);
    for(int j=0; j<i; ++j)
        glDisable(GL_LIGHTING);
}
